"""This module contains metadata about the project."""
title = 'fahrplan'
version = '1.1.0'
description = 'A SBB/CFF/FFS commandline based timetable client.'
author = 'Danilo Bargen and contributors'
author_email = 'mail@dbrgn.ch'
url = 'https://github.com/dbrgn/fahrplan'
keywords = 'fahrplan timetable sbb cff ffs public transport'
license = 'GPLv3'
